#!/usr/bin/perl

#-----------------------------------------------------------------------------
#  hdrFiddle.pl
#
#  Takes a C header file containing a number of structures and fiddles with
#  it such that:
#    - instances of #ifndef, #define and #endif are removed
#    - instances of 'Char' are converted to 'char'
#    - all field sizes in each structure are summed and a final field is
#      added which will have the effect of padding each structure to 1000
#      bytes
#    - edi_dc, edi_dd and edi_ds stuctures are removed
#-----------------------------------------------------------------------------
 
# First obtain the single command-line options, which is the input filename.
# If there are any more command-line arguments, they are ignored
$ifile = $ARGV[0];

# Try to open the given file, quitting if there is a problem
open( IN, $ifile ) || die "Cannot open input file '$ifile'. Quitting.\n";

# Create a corresponding output file, quitting if there is a problem. If the
# input file is called 'filename.h', then the output file will be called
# 'filename.h.out'. Note that if this file already exists, it will be over-
# written
$ofile = $ifile . ".out";
open( OUT, ">".$ofile ) || die "Cannot open output file '$ofile'. Quitting.\n";

# Now the main read-write loop
while( <IN> ) {
   # We have now read the next line in the input file and it can be referenced
   # using the variable $_. Initialise the output line to an empty string
   $outline = "";

   # First do a search-and-replace on Char
   $_ =~ s/ Char / char /;

   if( /^#ifndef/ || /^#define/ || /^#endif/ ) {
      # If the line starts with one of these strings, ignore it
   } elsif( /^typedef struct/ ) {
      # This denotes the start of a new struct definition. The first thing to
      # check is whether the struct that follows is one of the ones we are
      # going to ignore
      if( /edi_dc/ || /edi_dd/ || /edi_ds/ ) {
         # If it is, then flag the fact that we are currently ignoring all
         # input, so we know not to output the line later
         $ignore = 1;
      } else {
         # It's not one of the structs we wish to discard, so set the ignore
         # flag to false and save this line for outputting. Also set the
         # structSize counter to zero.
         $ignore     = 0;
         $structSize = 0;
         $outline    = $_;
      }
   } elsif( /}/ ) {
      # This denotes the end of a structure definition. If we are not
      # ignoring this block, we should calculate the pad field and add it
      # to the output
      if( !$ignore ) {
         $padsize = 1000-$structSize;

         # A bit of error checking here: If the padsize is zero or negative,
         # print an error message and die
         if( $padsize<1 ) { die "ERROR! Padsize is less than 1. Quitting!\n"; }

         # Build up the output line
         $outline = "    char pad${padsize}[${padsize}];       " .
                    "/* Pad string added by hdrFiddle */\n" . $_;
      }
   } elsif( !$ignore ) { 
      # If this line contains [n], then add 'n' to the current structSize
      if( /\[([0-9]+)\]/ ) {
         $structSize += $1;
      }
      # Whatever this line is, we aren't ignoring it - so copy it to the line
      # we will output
      $outline = $_;
   }

   # We now print the line to the output file
   print OUT $outline;
}

# Close input and output files. We're done!
close(IN);
close(OUT);
